@@ -1,13 +1,17 @@
+# mypy: disable-error-code="django-manager-missing"
+from __future__ import annotations
+
 from django.contrib.auth.models import Permission
 from django.core import validators
 from django.db import models
 from django.utils.translation import gettext_lazy
 
 from modeltranslation.manager import MultilingualManager
+from typing import Any
 
 
 class TestModel(models.Model):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
     text = models.TextField(blank=True, null=True)
     url = models.URLField(blank=True, null=True)
     email = models.EmailField(blank=True, null=True)
@@ -33,7 +37,7 @@ class ProxyTestModel(TestModel):
 
 
 class FallbackModel(models.Model):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
     text = models.TextField(blank=True, null=True)
     url = models.URLField(blank=True, null=True)
     email = models.EmailField(blank=True, null=True)
@@ -41,7 +45,7 @@ class FallbackModel(models.Model):
 
 
 class FallbackModel2(models.Model):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
     text = models.TextField(blank=True, null=True)
     url = models.URLField(blank=True, null=True)
     email = models.EmailField(blank=True, null=True)
@@ -51,21 +55,21 @@ class FallbackModel2(models.Model):
 
 
 class FileFieldsModel(models.Model):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
-    file = models.FileField(upload_to='modeltranslation_tests', null=True, blank=True)
-    file2 = models.FileField(upload_to='modeltranslation_tests')
-    image = models.ImageField(upload_to='modeltranslation_tests', null=True, blank=True)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
+    file = models.FileField(upload_to="modeltranslation_tests", null=True, blank=True)
+    file2 = models.FileField(upload_to="modeltranslation_tests")
+    image = models.ImageField(upload_to="modeltranslation_tests", null=True, blank=True)
 
 
 # ######### Foreign Key / OneToOneField / ManytoManyField testing
 
 
 class NonTranslated(models.Model):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
 
 
 class ForeignKeyModel(models.Model):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
     test = models.ForeignKey(
         TestModel,
         null=True,
@@ -97,7 +101,7 @@ class ForeignKeyModel(models.Model):
 
 
 class OneToOneFieldModel(models.Model):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
     test = models.OneToOneField(
         TestModel,
         null=True,
@@ -116,7 +120,7 @@ class OneToOneFieldModel(models.Model):
 
 
 class ManyToManyFieldModel(models.Model):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
     test = models.ManyToManyField(
         TestModel,
         related_name="m2m_test_ref",
@@ -124,8 +128,10 @@ class ManyToManyFieldModel(models.Model):
     self_call_1 = models.ManyToManyField("self")
     # test multiple self m2m
     self_call_2 = models.ManyToManyField("self")
-    through_model = models.ManyToManyField(TestModel, through="CustomThroughModel")
-    trans_through_model = models.ManyToManyField(
+    through_model = models.ManyToManyField[TestModel, "CustomThroughModel"](
+        TestModel, through="CustomThroughModel"
+    )
+    trans_through_model = models.ManyToManyField[TestModel, Any](
         TestModel, related_name="m2m_trans_through_model_ref", through="RegisteredThroughModel"
     )
     untrans = models.ManyToManyField(
@@ -184,8 +190,8 @@ class FancyDescriptor:
     def __get__(self, instance, owner):
         length = instance.__dict__[self.field.name]
         if length is None:
-            return ''
-        return 'a' * length
+            return ""
+        return "a" * length
 
     def __set__(self, obj, value):
         if isinstance(value, int):
@@ -198,7 +204,7 @@ class FancyDescriptor:
 
 class FancyField(models.PositiveIntegerField):
     def __init__(self, *args, **kwargs):
-        kwargs.setdefault('default', '')
+        kwargs.setdefault("default", "")
         super().__init__(*args, **kwargs)
 
     def contribute_to_class(self, cls, name):
@@ -222,41 +228,41 @@ class DescriptorModel(models.Model):
 
 
 class MultitableModelA(models.Model):
-    titlea = models.CharField(gettext_lazy('title a'), max_length=255)
+    titlea = models.CharField(gettext_lazy("title a"), max_length=255)
 
 
 class MultitableModelB(MultitableModelA):
-    titleb = models.CharField(gettext_lazy('title b'), max_length=255)
+    titleb = models.CharField(gettext_lazy("title b"), max_length=255)
 
 
 class MultitableModelC(MultitableModelB):
-    titlec = models.CharField(gettext_lazy('title c'), max_length=255)
+    titlec = models.CharField(gettext_lazy("title c"), max_length=255)
 
 
 class MultitableModelD(MultitableModelB):
-    titled = models.CharField(gettext_lazy('title d'), max_length=255)
+    titled = models.CharField(gettext_lazy("title d"), max_length=255)
 
 
 # ######### Abstract inheritance testing
 
 
 class AbstractModelA(models.Model):
-    titlea = models.CharField(gettext_lazy('title a'), max_length=255)
+    titlea = models.CharField(gettext_lazy("title a"), max_length=255)
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.titlea = 'title_a'
+        self.titlea = "title_a"
 
     class Meta:
         abstract = True
 
 
 class AbstractModelB(AbstractModelA):
-    titleb = models.CharField(gettext_lazy('title b'), max_length=255)
+    titleb = models.CharField(gettext_lazy("title b"), max_length=255)
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.titleb = 'title_b'
+        self.titleb = "title_b"
 
 
 # ######### Fields inheritance testing
@@ -341,12 +347,12 @@ class FilteredManager(MultilingualManager):
 
 
 class FilteredTestModel(models.Model):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
     objects = FilteredManager()
 
 
 class ForeignKeyFilteredModel(models.Model):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
     test = models.ForeignKey(
         FilteredTestModel,
         null=True,
@@ -356,28 +362,28 @@ class ForeignKeyFilteredModel(models.Model):
 
 
 class ManagerTestModel(models.Model):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
-    visits = models.IntegerField(gettext_lazy('visits'), default=0)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
+    visits = models.IntegerField(gettext_lazy("visits"), default=0)
     description = models.CharField(max_length=255, null=True)
 
     class Meta:
-        ordering = ('-visits',)
+        ordering = ("-visits",)
 
 
 class CustomManager(models.Manager):
     def get_queryset(self):
-        return super().get_queryset().filter(title__contains='a').exclude(description__contains='x')
+        return super().get_queryset().filter(title__contains="a").exclude(description__contains="x")
 
     def custom_qs(self):
         return super().get_queryset()
 
     def foo(self):
-        return 'bar'
+        return "bar"
 
 
 class CustomManagerTestModel(models.Model):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
-    description = models.CharField(max_length=255, null=True, db_column='xyz')
+    title = models.CharField(gettext_lazy("title"), max_length=255)
+    description = models.CharField(max_length=255, null=True, db_column="xyz")
     objects = CustomManager()
 
     another_mgr_name = CustomManager()
@@ -393,7 +399,7 @@ class CustomManager2(models.Manager):
 
 
 class CustomManager2TestModel(models.Model):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
     objects = CustomManager2()
 
 
@@ -412,13 +418,13 @@ class CustomManagerBaseModel(models.Model):
 
 
 class CustomManagerChildTestModel(CustomManagerBaseModel):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
 
-    objects = CustomManager2()
+    objects = CustomManager2()  # type: ignore[misc]
 
 
 class PlainChildTestModel(CustomManagerBaseModel):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
 
 
 # ######### Required fields testing
@@ -435,7 +441,7 @@ class RequiredModel(models.Model):
 
 
 class ConflictModel(models.Model):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
     title_de = models.IntegerField()
 
 
@@ -447,7 +453,7 @@ class AbstractConflictModelA(models.Model):
 
 
 class AbstractConflictModelB(AbstractConflictModelA):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
 
 
 class MultitableConflictModelA(models.Model):
@@ -455,7 +461,7 @@ class MultitableConflictModelA(models.Model):
 
 
 class MultitableConflictModelB(MultitableConflictModelA):
-    title = models.CharField(gettext_lazy('title'), max_length=255)
+    title = models.CharField(gettext_lazy("title"), max_length=255)
 
 
 # ######### Complex M2M with abstract classes and custom managers
@@ -488,8 +494,8 @@ class ModelX(AbstractModelX):
 
 
 class AbstractModelXY(models.Model):
-    model_x = models.ForeignKey('ModelX', on_delete=models.CASCADE)
-    model_y = models.ForeignKey('ModelY', on_delete=models.CASCADE)
+    model_x = models.ForeignKey("ModelX", on_delete=models.CASCADE)
+    model_y = models.ForeignKey("ModelY", on_delete=models.CASCADE)
 
     class Meta:
         abstract = True
@@ -505,7 +511,7 @@ class CustomManagerY(models.Manager):
 
 class AbstractModelY(models.Model):
     title = models.CharField(max_length=255)
-    xs = models.ManyToManyField('ModelX', through='ModelXY')
+    xs = models.ManyToManyField[ModelX, ModelXY]("ModelX", through="ModelXY")
     objects = CustomManagerY()
 
     class Meta:
