@@ -4,7 +4,7 @@ from .test_app.models import Other
 
 
 class OtherTranslationOptions(TranslationOptions):
-    fields = ('name',)
+    fields = ("name",)
 
 
 translator.register(Other, OtherTranslationOptions)
