@@ -10,18 +10,18 @@ from modeltranslation.translator import TranslationOptions, register, translator
 @register(models.TestModel)
 class TestTranslationOptions(TranslationOptions):
     fields = (
-        'title',
-        'text',
-        'url',
-        'email',
-        'dynamic_default',
+        "title",
+        "text",
+        "url",
+        "email",
+        "dynamic_default",
     )
-    empty_values = ''
+    empty_values = ""
 
 
 @register(models.UniqueNullableModel)
 class UniqueNullableTranslationOptions(TranslationOptions):
-    fields = ('title',)
+    fields = ("title",)
 
 
 # ######### Proxy model testing
@@ -30,10 +30,10 @@ class UniqueNullableTranslationOptions(TranslationOptions):
 @register(models.ProxyTestModel)
 class ProxyTestTranslationOptions(TranslationOptions):
     fields = (
-        'title',
-        'text',
-        'url',
-        'email',
+        "title",
+        "text",
+        "url",
+        "email",
     )
 
 
@@ -42,20 +42,20 @@ class ProxyTestTranslationOptions(TranslationOptions):
 
 @register(models.FallbackModel)
 class FallbackModelTranslationOptions(TranslationOptions):
-    fields = ('title', 'text', 'url', 'email', 'description')
+    fields = ("title", "text", "url", "email", "description")
     fallback_values = "fallback"
 
 
 @register(models.FallbackModel2)
 class FallbackModel2TranslationOptions(TranslationOptions):
     fields = (
-        'title',
-        'text',
-        'url',
-        'email',
+        "title",
+        "text",
+        "url",
+        "email",
     )
-    fallback_values = {'text': gettext_lazy('Sorry, translation is not available.')}
-    fallback_undefined = {'title': 'no title'}
+    fallback_values = {"text": gettext_lazy("Sorry, translation is not available.")}
+    fallback_undefined = {"title": "no title"}
 
 
 # ######### File fields testing
@@ -64,10 +64,10 @@ class FallbackModel2TranslationOptions(TranslationOptions):
 @register(models.FileFieldsModel)
 class FileFieldsModelTranslationOptions(TranslationOptions):
     fields = (
-        'title',
-        'file',
-        'file2',
-        'image',
+        "title",
+        "file",
+        "file2",
+        "image",
     )
 
 
@@ -77,32 +77,32 @@ class FileFieldsModelTranslationOptions(TranslationOptions):
 @register(models.ForeignKeyModel)
 class ForeignKeyModelTranslationOptions(TranslationOptions):
     fields = (
-        'title',
-        'test',
-        'optional',
-        'hidden',
-        'non',
+        "title",
+        "test",
+        "optional",
+        "hidden",
+        "non",
     )
 
 
 @register(models.OneToOneFieldModel)
 class OneToOneFieldModelTranslationOptions(TranslationOptions):
     fields = (
-        'title',
-        'test',
-        'optional',
-        'non',
+        "title",
+        "test",
+        "optional",
+        "non",
     )
 
 
 @register(models.FilteredTestModel)
 class FilteredTestModelTranslationOptions(TranslationOptions):
-    fields = ('title',)
+    fields = ("title",)
 
 
 @register(models.ForeignKeyFilteredModel)
 class ForeignKeyFilteredModelTranslationOptions(TranslationOptions):
-    fields = ('title',)
+    fields = ("title",)
 
 
 @register(models.ManyToManyFieldModel)
@@ -120,7 +120,7 @@ class ManyToManyFieldModelTranslationOptions(TranslationOptions):
 
 @register(models.RegisteredThroughModel)
 class RegisteredThroughModelTranslationOptions(TranslationOptions):
-    fields = ('title',)
+    fields = ("title",)
 
 
 # ######### Custom fields testing
@@ -129,21 +129,21 @@ class RegisteredThroughModelTranslationOptions(TranslationOptions):
 @register(models.OtherFieldsModel)
 class OtherFieldsModelTranslationOptions(TranslationOptions):
     fields = (
-        'int',
-        'boolean',
-        'float',
-        'decimal',
-        'genericip',
-        'date',
-        'datetime',
-        'time',
-        'json',
+        "int",
+        "boolean",
+        "float",
+        "decimal",
+        "genericip",
+        "date",
+        "datetime",
+        "time",
+        "json",
     )
 
 
 @register(models.DescriptorModel)
 class DescriptorModelTranslationOptions(TranslationOptions):
-    fields = ('trans',)
+    fields = ("trans",)
 
 
 # ######### Multitable inheritance testing
@@ -151,17 +151,17 @@ class DescriptorModelTranslationOptions(TranslationOptions):
 
 @register(models.MultitableModelA)
 class MultitableModelATranslationOptions(TranslationOptions):
-    fields = ('titlea',)
+    fields = ("titlea",)
 
 
 @register(models.MultitableModelB)
 class MultitableModelBTranslationOptions(TranslationOptions):
-    fields = ('titleb',)
+    fields = ("titleb",)
 
 
 @register(models.MultitableModelC)
 class MultitableModelCTranslationOptions(TranslationOptions):
-    fields = ('titlec',)
+    fields = ("titlec",)
 
 
 # ######### Abstract inheritance testing
@@ -169,31 +169,31 @@ class MultitableModelCTranslationOptions(TranslationOptions):
 
 @register(models.AbstractModelA)
 class AbstractModelATranslationOptions(TranslationOptions):
-    fields = ('titlea',)
+    fields = ("titlea",)
 
 
 @register(models.AbstractModelB)
 class AbstractModelBTranslationOptions(TranslationOptions):
-    fields = ('titleb',)
+    fields = ("titleb",)
 
 
 # ######### Fields inheritance testing
 
 
 class SluggedTranslationOptions(TranslationOptions):
-    fields = ('slug',)
+    fields = ("slug",)
 
 
 class MetaDataTranslationOptions(TranslationOptions):
-    fields = ('keywords',)
+    fields = ("keywords",)
 
 
 class RichTextTranslationOptions(TranslationOptions):
-    fields = ('content',)
+    fields = ("content",)
 
 
 class PageTranslationOptions(TranslationOptions):
-    fields = ('title',)
+    fields = ("title",)
 
 
 # BasePage left unregistered intentionally.
@@ -210,7 +210,7 @@ translator.register(models.RichTextPage)
 
 @register(models.ManagerTestModel)
 class ManagerTestModelTranslationOptions(TranslationOptions):
-    fields = ('title', 'visits', 'description')
+    fields = ("title", "visits", "description")
 
 
 @register(
@@ -222,32 +222,32 @@ class ManagerTestModelTranslationOptions(TranslationOptions):
     ]
 )
 class CustomManagerTestModelTranslationOptions(TranslationOptions):
-    fields = ('title',)
+    fields = ("title",)
 
 
 # ######### TranslationOptions field inheritance testing
 
 
 class FieldInheritanceATranslationOptions(TranslationOptions):
-    fields = ['titlea']
+    fields = ["titlea"]
 
 
 class FieldInheritanceBTranslationOptions(FieldInheritanceATranslationOptions):
-    fields = ['titleb']
+    fields = ["titleb"]
 
 
 class FieldInheritanceCTranslationOptions(FieldInheritanceBTranslationOptions):
-    fields = ['titlec']
+    fields = ["titlec"]
 
 
 class FieldInheritanceDTranslationOptions(FieldInheritanceBTranslationOptions):
-    fields = ('titled',)
+    fields = ["titled"]
 
 
 class FieldInheritanceETranslationOptions(
     FieldInheritanceCTranslationOptions, FieldInheritanceDTranslationOptions
 ):
-    fields = ('titlee',)
+    fields = ["titlee"]
 
 
 # ######### Integration testing
@@ -255,7 +255,7 @@ class FieldInheritanceETranslationOptions(
 
 @register(models.ThirdPartyRegisteredModel)
 class ThirdPartyTranslationOptions(TranslationOptions):
-    fields = ('name',)
+    fields = ("name",)
 
 
 # ######### Admin testing
@@ -264,14 +264,14 @@ class ThirdPartyTranslationOptions(TranslationOptions):
 @register(models.GroupFieldsetsModel)
 class GroupFieldsetsTranslationOptions(TranslationOptions):
     fields = (
-        'title',
-        'text',
+        "title",
+        "text",
     )
 
 
 @register(models.NameModel)
 class NameTranslationOptions(TranslationOptions):
-    fields = ('firstname', 'lastname', 'slug2')
+    fields = ("firstname", "lastname", "slug2")
 
 
 # ######### Required fields testing
@@ -279,13 +279,16 @@ class NameTranslationOptions(TranslationOptions):
 
 @register(models.RequiredModel)
 class RequiredTranslationOptions(TranslationOptions):
-    fields = ('non_req', 'req', 'req_reg', 'req_en_reg')
+    fields = ("non_req", "req", "req_reg", "req_en_reg")
     required_languages = {
-        'en': (
-            'req_reg',
-            'req_en_reg',
+        "en": (
+            # We include `non_req` field here, to test that it's `blank` attribute is preserved,
+            # even when languages is required.
+            "non_req",
+            "req_reg",
+            "req_en_reg",
         ),
-        'default': ('req_reg',),  # for all other languages
+        "default": ("req_reg",),  # for all other languages
     }
 
 
@@ -294,12 +297,12 @@ class RequiredTranslationOptions(TranslationOptions):
 
 @register(models.ModelX)
 class ModelXOptions(TranslationOptions):
-    fields = ('name',)
+    fields = ("name",)
 
 
 @register(models.ModelY)
 class ModelYOptions(TranslationOptions):
-    fields = ('title',)
+    fields = ("title",)
 
 
 # ######### 3-rd party with custom manager
@@ -307,10 +310,10 @@ class ModelYOptions(TranslationOptions):
 
 @register(Group)
 class GroupTranslationOptions(TranslationOptions):
-    fields = ('name',)
+    fields = ("name",)
 
 
 @register(InheritedPermission)
 class InheritedPermissionOptions(TranslationOptions):
-    fields = ('translated_var',)
+    fields = ("translated_var",)
     required_languages = [x[0] for x in settings.LANGUAGES]
