@@ -1,9 +1,19 @@
+from __future__ import annotations
+
 from functools import partial
-from typing import Callable, Iterable
+from typing import Any, Callable, ClassVar, cast
+from collections.abc import Collection, Iterable, Sequence
 
-import django
 from django.core.exceptions import ImproperlyConfigured
-from django.db.models import ForeignKey, Manager, ManyToManyField, OneToOneField, options
+from django.db.models import (
+    Field,
+    ForeignKey,
+    Manager,
+    ManyToManyField,
+    Model,
+    OneToOneField,
+    options,
+)
 from django.db.models.base import ModelBase
 from django.db.models.signals import post_init
 from django.utils.functional import cached_property
@@ -11,20 +21,39 @@ from django.utils.functional import cached_property
 from modeltranslation import settings as mt_settings
 from modeltranslation.fields import (
     NONE,
-    LanguageCacheSingleObjectDescriptor,
     TranslatedManyToManyDescriptor,
     TranslatedRelationIdDescriptor,
     TranslationFieldDescriptor,
+    TranslationField,
     create_translation_field,
 )
 from modeltranslation.manager import (
     MultilingualManager,
     MultilingualQuerysetManager,
-    append_translated,
     rewrite_lookup_key,
 )
 from modeltranslation.thread_context import auto_populate_mode
-from modeltranslation.utils import build_localized_fieldname, parse_field
+from modeltranslation.utils import (
+    build_localized_fieldname,
+    parse_field,
+    get_language,
+)
+
+# Re-export the decorator for convenience
+from modeltranslation.decorators import register
+
+from ._compat import is_hidden, build_refresh_from_db
+from ._typing import _ListOrTuple
+
+__all__ = [
+    "AlreadyRegistered",
+    "DescendantRegistered",
+    "NotRegistered",
+    "TranslationOptions",
+    "Translator",
+    "register",
+    "translator",
+]
 
 
 class AlreadyRegistered(Exception):
@@ -44,12 +73,14 @@ class FieldsAggregationMetaClass(type):
     Metaclass to handle custom inheritance of fields between classes.
     """
 
-    def __new__(cls, name, bases, attrs):
-        attrs['fields'] = set(attrs.get('fields', ()))
+    fields: Sequence[str]
+
+    def __new__(cls, name: str, bases: tuple[type, ...], attrs: dict[str, Any]) -> type:
+        attrs["fields"] = set(attrs.get("fields", ()))
         for base in bases:
             if isinstance(base, FieldsAggregationMetaClass):
-                attrs['fields'].update(base.fields)
-        attrs['fields'] = tuple(attrs['fields'])
+                attrs["fields"].update(base.fields)
+        attrs["fields"] = tuple(attrs["fields"])
         return super().__new__(cls, name, bases, attrs)
 
 
@@ -61,11 +92,11 @@ class TranslationOptions(metaclass=FieldsAggregationMetaClass):
     attributes.
 
     Options instances hold info about translatable fields for a model and its
-    superclasses. The ``local_fields`` and ``fields`` attributes are mappings
+    superclasses. The ``local_fields`` and ``all_fields`` attributes are mappings
     from fields to sets of their translation fields; ``local_fields`` contains
     only those fields that are handled in the model's database table (those
     inherited from abstract superclasses, unless there is a concrete superclass
-    in between in the inheritance chain), while ``fields`` also includes fields
+    in between in the inheritance chain), while ``all_fields`` also includes fields
     inherited from concrete supermodels (giving all translated fields available
     on a model).
 
@@ -74,68 +105,74 @@ class TranslationOptions(metaclass=FieldsAggregationMetaClass):
     ``related_fields`` contains names of reverse lookup fields.
     """
 
-    required_languages = ()
+    required_languages: ClassVar[_ListOrTuple[str] | dict[str, _ListOrTuple[str]]] = (
+        mt_settings.REQUIRED_LANGUAGES
+    )
 
-    def __init__(self, model):
+    def __init__(self, model: Model) -> None:
         """
         Create fields dicts without any translation fields.
         """
         self.model = model
         self.registered = False
         self.related = False
-        self.local_fields = {f: set() for f in self.fields}
-        self.fields = {f: set() for f in self.fields}
-        self.related_fields = []
+        self.local_fields: dict[str, set[TranslationField]] = {f: set() for f in self.fields}
+        self.all_fields: dict[str, set[TranslationField]] = {f: set() for f in self.fields}
+        self.related_fields: list[str] = []
 
-    def validate(self):
+    def validate(self) -> None:
         """
         Perform options validation.
         """
         # TODO: at the moment only required_languages is validated.
         # Maybe check other options as well?
-        if self.required_languages:
-            if isinstance(self.required_languages, (tuple, list)):
-                self._check_languages(self.required_languages)
+        if value := self.required_languages:
+            if isinstance(value, (tuple, list)):
+                self._check_languages(value)
             else:
-                self._check_languages(self.required_languages.keys(), extra=('default',))
-                for fieldnames in self.required_languages.values():
-                    if any(f not in self.fields for f in fieldnames):
+                self._check_languages(value.keys(), extra=("default",))
+                for fieldnames in value.values():
+                    if any(f not in self.all_fields for f in fieldnames):
                         raise ImproperlyConfigured(
-                            'Fieldname in required_languages which is not in fields option.'
+                            "Fieldname in required_languages which is not in fields option."
                         )
 
-    def _check_languages(self, languages, extra=()):
+    def _check_languages(
+        self,
+        languages: Collection[str],
+        extra: tuple[str, ...] = (),
+    ) -> None:
         correct = list(mt_settings.AVAILABLE_LANGUAGES) + list(extra)
         if any(lang not in correct for lang in languages):
             raise ImproperlyConfigured(
-                'Language in required_languages which is not in AVAILABLE_LANGUAGES.'
+                "Language in required_languages which is not in AVAILABLE_LANGUAGES."
             )
 
-    def update(self, other):
+    def update(self, other: TranslationOptions):
         """
         Update with options from a superclass.
         """
         if other.model._meta.abstract:
             self.local_fields.update(other.local_fields)
-        self.fields.update(other.fields)
+        self.all_fields.update(other.all_fields)
 
-    def add_translation_field(self, field, translation_field):
+    def add_translation_field(self, field: str, translation_field):
         """
         Add a new translation field to both fields dicts.
         """
         self.local_fields[field].add(translation_field)
-        self.fields[field].add(translation_field)
+        self.all_fields[field].add(translation_field)
 
-    def get_field_names(self):
+    def get_field_names(self) -> list[str]:
         """
         Return name of all fields that can be used in filtering.
         """
-        return list(self.fields.keys()) + self.related_fields
+        return list(self.all_fields.keys()) + self.related_fields
 
-    def __str__(self):
+    def __str__(self) -> str:
         local = tuple(self.local_fields.keys())
-        inherited = tuple(set(self.fields.keys()) - set(local))
-        return '%s: %s + %s' % (self.__class__.__name__, local, inherited)
+        inherited = tuple(set(self.all_fields.keys()) - set(local))
+        return "%s: %s + %s" % (self.__class__.__name__, local, inherited)
 
 
 class MultilingualOptions(options.Options):
@@ -146,14 +183,14 @@ class MultilingualOptions(options.Options):
         return manager
 
 
-def add_translation_fields(model, opts):
+def add_translation_fields(model: type[Model], opts: TranslationOptions) -> None:
     """
     Monkey patches the original model class to provide additional fields for
     every language.
 
     Adds newly created translation fields to the given translation options.
     """
-    model_empty_values = getattr(opts, 'empty_values', NONE)
+    model_empty_values = getattr(opts, "empty_values", NONE)
     for field_name in opts.local_fields.keys():
         field_empty_value = parse_field(model_empty_values, field_name, NONE)
         for lang in mt_settings.AVAILABLE_LANGUAGES:
@@ -168,7 +205,7 @@ def add_translation_fields(model, opts):
             if hasattr(model, localized_field_name):
                 # Check if are not dealing with abstract field inherited.
                 for cls in model.__mro__:
-                    if hasattr(cls, '_meta') and cls.__dict__.get(localized_field_name, None):
+                    if hasattr(cls, "_meta") and cls.__dict__.get(localized_field_name, None):
                         cls_opts = translator._get_options_for_model(cls)
                         if not cls._meta.abstract or field_name not in cls_opts.local_fields:
                             raise ValueError(
@@ -204,7 +241,7 @@ def patch_manager_class(manager):
             def deconstruct(self):
                 return (
                     False,  # as_manager
-                    '%s.%s' % (self._old_module, self._old_class),  # manager_class
+                    "%s.%s" % (self._old_module, self._old_class),  # manager_class
                     None,  # qs_class
                     self._constructor_args[0],  # args
                     self._constructor_args[1],  # kwargs
@@ -229,7 +266,7 @@ def patch_manager_class(manager):
         manager.__class__ = NewMultilingualManager
 
 
-def add_manager(model):
+def add_manager(model: type[Model]) -> None:
     """
     Monkey patches the original model to use MultilingualManager instead of
     default managers (not only ``objects``, but also every manager defined and inherited).
@@ -240,7 +277,7 @@ def add_manager(model):
         return
     # Make all managers local for this model to fix patching parent model managers
     added = set(model._meta.managers) - set(model._meta.local_managers)
-    model._meta.local_managers = model._meta.managers
+    model._meta.local_managers = model._meta.managers  # type: ignore[assignment]
 
     for current_manager in model._meta.local_managers:
         prev_class = current_manager.__class__
@@ -262,7 +299,7 @@ def add_manager(model):
     model._meta._expire_cache()
 
 
-def patch_constructor(model):
+def patch_constructor(model: type[Model]) -> None:
     """
     Monkey patches the original model to rewrite fields names in __init__
     """
@@ -280,19 +317,19 @@ def patch_constructor(model):
     model.__init__ = new_init
 
 
-def delete_mt_init(sender, instance, **kwargs):
-    if hasattr(instance, '_mt_init'):
+def delete_mt_init(sender: type[Model], instance: Model, **kwargs: Any) -> None:
+    if hasattr(instance, "_mt_init"):
         del instance._mt_init
 
 
-def patch_clean_fields(model):
+def patch_clean_fields(model: type[Model]):
     """
     Patch clean_fields method to handle different form types submission.
     """
     old_clean_fields = model.clean_fields
 
-    def new_clean_fields(self, exclude=None):
-        if hasattr(self, '_mt_form_pending_clear'):
+    def new_clean_fields(self, exclude: Collection[str] | None = None) -> None:
+        if hasattr(self, "_mt_form_pending_clear") and exclude is not None:
             # Some form translation fields has been marked as clearing value.
             # Check if corresponding translated field was also saved (not excluded):
             # - if yes, it seems like form for MT-unaware app. Ignore clearing (left value from
@@ -303,58 +340,52 @@ def patch_clean_fields(model):
                 orig_field_name = field.translated_field.name
                 if orig_field_name in exclude:
                     field.save_form_data(self, value, check=False)
-            delattr(self, '_mt_form_pending_clear')
+            delattr(self, "_mt_form_pending_clear")
         try:
-            setattr(self, '_mt_disable', True)
+            setattr(self, "_mt_disable", True)
             old_clean_fields(self, exclude)
         finally:
-            setattr(self, '_mt_disable', False)
+            setattr(self, "_mt_disable", False)
 
     model.clean_fields = new_clean_fields
 
 
-def patch_get_deferred_fields(model):
+def patch_get_deferred_fields(model: type[Model]) -> None:
     """
     Django >= 1.8: patch detecting deferred fields. Crucial for only/defer to work.
     """
-    if not hasattr(model, 'get_deferred_fields'):
+    if not hasattr(model, "get_deferred_fields"):
         return
     old_get_deferred_fields = model.get_deferred_fields
 
-    def new_get_deferred_fields(self):
+    def new_get_deferred_fields(self) -> set[str]:
         sup = old_get_deferred_fields(self)
-        if hasattr(self, '_fields_were_deferred'):
+        if hasattr(self, "_fields_were_deferred"):
             sup.update(self._fields_were_deferred)
         return sup
 
     model.get_deferred_fields = new_get_deferred_fields
 
 
-def patch_refresh_from_db(model):
+def patch_refresh_from_db(model: type[Model]) -> None:
     """
     Django >= 1.10: patch refreshing deferred fields. Crucial for only/defer to work.
     """
-    if not hasattr(model, 'refresh_from_db'):
+    if not hasattr(model, "refresh_from_db"):
         return
-    old_refresh_from_db = model.refresh_from_db
 
-    def new_refresh_from_db(self, using=None, fields=None):
-        if fields is not None:
-            fields = append_translated(self.__class__, fields)
-        return old_refresh_from_db(self, using, fields)
-
-    model.refresh_from_db = new_refresh_from_db
+    model.refresh_from_db = build_refresh_from_db(model.refresh_from_db)
 
 
-def delete_cache_fields(model):
+def delete_cache_fields(model: type[Model]) -> None:
     opts = model._meta
     cached_attrs = (
-        '_field_cache',
-        '_field_name_cache',
-        '_name_map',
-        'fields',
-        'concrete_fields',
-        'local_concrete_fields',
+        "_field_cache",
+        "_field_name_cache",
+        "_name_map",
+        "fields",
+        "concrete_fields",
+        "local_concrete_fields",
     )
     for attr in cached_attrs:
         try:
@@ -362,11 +393,11 @@ def delete_cache_fields(model):
         except AttributeError:
             pass
 
-    if hasattr(model._meta, '_expire_cache'):
+    if hasattr(model._meta, "_expire_cache"):
         model._meta._expire_cache()
 
 
-def populate_translation_fields(sender, kwargs):
+def populate_translation_fields(sender: type[Model], kwargs: Any):
     """
     When models are created or loaded from fixtures, replicates values
     provided for translatable fields to some / all empty translation fields,
@@ -396,21 +427,21 @@ def populate_translation_fields(sender, kwargs):
         return
     if populate is True:
         # What was meant by ``True`` is now called ``all``.
-        populate = 'all'
+        populate = "all"
 
     opts = translator.get_options_for_model(sender)
     for key, val in list(kwargs.items()):
-        if key in opts.fields:
-            if populate == 'all':
+        if key in opts.all_fields:
+            if populate == "all":
                 # Set the value for every language.
-                for translation_field in opts.fields[key]:
+                for translation_field in opts.all_fields[key]:
                     kwargs.setdefault(translation_field.name, val)
-            elif populate == 'default':
+            elif populate == "default":
                 default = build_localized_fieldname(key, mt_settings.DEFAULT_LANGUAGE)
                 kwargs.setdefault(default, val)
-            elif populate == 'required':
+            elif populate == "required":
                 default = build_localized_fieldname(key, mt_settings.DEFAULT_LANGUAGE)
-                if not sender._meta.get_field(key).null:
+                if not sender._meta.get_field(key).null:  # type: ignore[union-attr]
                     kwargs.setdefault(default, val)
             else:
                 raise AttributeError("Unknown population mode '%s'." % populate)
@@ -422,17 +453,21 @@ def patch_related_object_descriptor_caching(ro_descriptor):
     language-aware caching.
     """
 
-    class NewSingleObjectDescriptor(LanguageCacheSingleObjectDescriptor, ro_descriptor.__class__):
-        pass
+    class NewRelated(ro_descriptor.related.__class__):
+        def get_cache_name(self) -> str:
+            """
+            Used in django > 2.x
+            """
+            return self.cache_name
 
-    if django.VERSION[0] >= 2:
-        ro_descriptor.related.get_cache_name = partial(
-            NewSingleObjectDescriptor.get_cache_name,
-            ro_descriptor,
-        )
+        @property
+        def cache_name(self):
+            """
+            Used in django >= 5.1
+            """
+            return build_localized_fieldname(self.get_accessor_name(), get_language())
 
-    ro_descriptor.accessor = ro_descriptor.related.get_accessor_name()
-    ro_descriptor.__class__ = NewSingleObjectDescriptor
+    ro_descriptor.related.__class__ = NewRelated
 
 
 class Translator:
@@ -441,13 +476,18 @@ class Translator:
     registered with the Translator using the register() method.
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         # All seen models (model class -> ``TranslationOptions`` instance).
-        self._registry = {}
+        self._registry: dict[type[Model], TranslationOptions] = {}
         # List of funcs to execute after all imports are done.
-        self._lazy_operations: Iterable[Callable] = []
+        self._lazy_operations: list[Callable[..., Any]] = []
 
-    def register(self, model_or_iterable, opts_class=None, **options):
+    def register(
+        self,
+        model_or_iterable: type[Model] | Iterable[type[Model]],
+        opts_class: type[TranslationOptions] | None = None,
+        **options: Any,
+    ) -> None:
         """
         Registers the given model(s) with the given translation options.
 
@@ -491,7 +531,7 @@ class Translator:
                 self._registry[model].registered = False
                 raise
 
-    def _register_single_model(self, model, opts):
+    def _register_single_model(self, model: type[Model], opts: TranslationOptions) -> None:
         # Now, when all fields are initialized and inherited, validate configuration.
         opts.validate()
 
@@ -532,11 +572,11 @@ class Translator:
         patch_refresh_from_db(model)
 
         # Substitute original field with descriptor
-        model_fallback_languages = getattr(opts, 'fallback_languages', None)
-        model_fallback_values = getattr(opts, 'fallback_values', NONE)
-        model_fallback_undefined = getattr(opts, 'fallback_undefined', NONE)
+        model_fallback_languages = getattr(opts, "fallback_languages", None)
+        model_fallback_values = getattr(opts, "fallback_values", NONE)
+        model_fallback_undefined = getattr(opts, "fallback_undefined", NONE)
         for field_name in opts.local_fields.keys():
-            field = model._meta.get_field(field_name)
+            field = cast(Field, model._meta.get_field(field_name))
             field_fallback_value = parse_field(model_fallback_values, field_name, NONE)
             field_fallback_undefined = parse_field(model_fallback_undefined, field_name, NONE)
             descriptor = TranslationFieldDescriptor(
@@ -559,7 +599,7 @@ class Translator:
                 setattr(model, field.get_attname(), desc)
 
                 # Set related field names on other model
-                if not field.remote_field.is_hidden():
+                if not is_hidden(field.remote_field):
                     other_opts = self._get_options_for_model(field.remote_field.model)
                     other_opts.related = True
                     other_opts.related_fields.append(field.related_query_name())
@@ -569,11 +609,12 @@ class Translator:
             if isinstance(field, OneToOneField):
                 # Fix translated_field caching for SingleRelatedObjectDescriptor
                 sro_descriptor = getattr(
-                    field.remote_field.model, field.remote_field.get_accessor_name()
+                    field.remote_field.model,
+                    field.remote_field.get_accessor_name(),
                 )
                 patch_related_object_descriptor_caching(sro_descriptor)
 
-    def unregister(self, model_or_iterable):
+    def unregister(self, model_or_iterable: type[Model] | Iterable[type[Model]]) -> None:
         """
         Unregisters the given model(s).
 
@@ -600,7 +641,7 @@ class Translator:
                     )
                 del self._registry[desc]
 
-    def get_registered_models(self, abstract=True):
+    def get_registered_models(self, abstract: bool = True) -> list[type[Model]]:
         """
         Returns a list of all registered models, or just concrete
         registered models.
@@ -611,7 +652,9 @@ class Translator:
             if opts.registered and (not model._meta.abstract or abstract)
         ]
 
-    def _get_options_for_model(self, model, opts_class=None, **options):
+    def _get_options_for_model(
+        self, model: type[Model], opts_class: type[TranslationOptions] | None = None, **options: Any
+    ) -> TranslationOptions:
         """
         Returns an instance of translation options with translated fields
         defined for the ``model`` and inherited from superclasses.
@@ -628,7 +671,7 @@ class Translator:
             # Fields for translation may be inherited from abstract
             # superclasses, so we need to look at all parents.
             for base in model.__bases__:
-                if not hasattr(base, '_meta'):
+                if not hasattr(base, "_meta"):
                     # Things without _meta aren't functional models, so they're
                     # uninteresting parents.
                     continue
@@ -640,7 +683,7 @@ class Translator:
 
         return self._registry[model]
 
-    def get_options_for_model(self, model) -> TranslationOptions:
+    def get_options_for_model(self, model: type[Model]) -> TranslationOptions:
         """
         Thin wrapper around ``_get_options_for_model`` to preserve the
         semantic of throwing exception for models not directly registered.
@@ -648,7 +691,7 @@ class Translator:
         opts = self._get_options_for_model(model)
         if not opts.registered and not opts.related:
             raise NotRegistered(
-                'The model "%s" is not registered for ' 'translation' % model.__name__
+                'The model "%s" is not registered for ' "translation" % model.__name__
             )
         return opts
 
@@ -656,13 +699,9 @@ class Translator:
         while self._lazy_operations:
             self._lazy_operations.pop(0)(translator=self)
 
-    def lazy_operation(self, func: Callable, *args, **kwargs) -> None:
+    def lazy_operation(self, func: Callable[..., Any], *args: Any, **kwargs: Any) -> None:
         self._lazy_operations.append(partial(func, *args, **kwargs))
 
 
 # This global object represents the singleton translator object
 translator = Translator()
-
-
-# Re-export the decorator for convenience
-from modeltranslation.decorators import register  # NOQA re-export
